@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -70,7 +71,8 @@ public slots:
 protected slots:
     void slotAddFolder();
     void slotEnableCurrentFolder();
-    void slotSyncCurrentFolderNow();
+    void slotScheduleCurrentFolder();
+    void slotForceSyncCurrentFolder();
     void slotRemoveCurrentFolder();
     void slotOpenCurrentFolder();
     void slotFolderWizardAccepted();
