@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -280,9 +281,12 @@ void AccountState::slotCredentialsFetched(AbstractCredentials* credentials)
 
     _waitingForNewCredentials = false;
 
-    // When new credentials become available we always want to restart the
-    // connection validation, even if it's currently running.
-    delete _connectionValidator;
+    if (_connectionValidator) {
+        // When new credentials become available we always want to restart the
+        // connection validation, even if it's currently running.
+        _connectionValidator->deleteLater();
+        _connectionValidator = 0;
+    }
 
     checkConnectivity();
 }
@@ -297,16 +301,19 @@ void AccountState::slotCredentialsAsked(AbstractCredentials* credentials)
         return;
     }
 
-    // When new credentials become available we always want to restart the
-    // connection validation, even if it's currently running.
-    delete _connectionValidator;
+    if (_connectionValidator) {
+        // When new credentials become available we always want to restart the
+        // connection validation, even if it's currently running.
+        _connectionValidator->deleteLater();
+        _connectionValidator = 0;
+    }
 
     checkConnectivity();
 }
 
 std::unique_ptr<QSettings> AccountState::settings()
 {
-    auto s = _account->settingsWithGroup(QLatin1String("Accounts"));
+    auto s = Utility::settingsWithGroup(QLatin1String("Accounts"));
     s->beginGroup(_account->id());
     return s;
 }
