@@ -23,6 +23,7 @@
 #ifndef TOKEN_AUTH_ONLY
 #include <QtGui>
 #endif
+#include <QSslSocket>
 
 #include "owncloudtheme.h"
 
@@ -115,11 +116,11 @@ QIcon Theme::trayFolderIcon( const QString& backend ) const
  * helper to load a icon from either the icon theme the desktop provides or from
  * the apps Qt resources.
  */
-QIcon Theme::themeIcon( const QString& name, bool sysTray ) const
+QIcon Theme::themeIcon( const QString& name, bool sysTray, bool sysTrayMenuVisible ) const
 {
     QString flavor;
     if (sysTray) {
-        flavor = systrayIconFlavor(_mono);
+        flavor = systrayIconFlavor(_mono, sysTrayMenuVisible);
     } else {
         flavor = QLatin1String("colored");
     }
@@ -158,6 +159,14 @@ QIcon Theme::themeIcon( const QString& name, bool sysTray ) const
         }
     }
 
+    #ifdef Q_OS_MAC
+    #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+    // This defines the icon as a template and enables automatic macOS color handling
+    // See https://bugreports.qt.io/browse/QTBUG-42109
+    cached.setIsMask(_mono && sysTray && !sysTrayMenuVisible);
+    #endif
+    #endif
+
     return cached;
 }
 
@@ -227,11 +236,18 @@ QString Theme::defaultClientFolder() const
     return appName();
 }
 
-QString Theme::systrayIconFlavor(bool mono) const
+QString Theme::systrayIconFlavor(bool mono, bool sysTrayMenuVisible ) const
 {
+    Q_UNUSED(sysTrayMenuVisible)
     QString flavor;
     if (mono) {
         flavor = Utility::hasDarkSystray() ? QLatin1String("white") : QLatin1String("black");
+
+        #ifdef Q_OS_MAC
+        if (sysTrayMenuVisible) {
+            flavor = QLatin1String("white");
+        }
+        #endif
     } else {
         flavor = QLatin1String("colored");
     }
@@ -260,6 +276,15 @@ qint64 Theme::newBigFolderSizeLimit() const
     return 500;
 }
 
+bool Theme::wizardHideExternalStorageConfirmationCheckbox() const
+{
+    return false;
+}
+
+bool Theme::wizardHideFolderSizeLimitCheckbox() const
+{
+    return false;
+}
 
 QString Theme::gitSHA1() const
 {
@@ -273,8 +298,13 @@ QString Theme::gitSHA1() const
                    " on %3, %4 using Qt %5, %6</small></p>")
             .arg(githubPrefix+gitSha1).arg(gitSha1.left(6))
             .arg(__DATE__).arg(__TIME__)
-            .arg(QT_VERSION_STR)
-            .arg(QString::fromAscii(OPENSSL_VERSION_TEXT));
+            .arg(QString::fromAscii(qVersion()))
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+            .arg(QSslSocket::sslLibraryVersionString());
+#else
+            .arg(QCoreApplication::translate("ownCloudTheme::about()", "built with %1").arg(
+                     QString::fromAscii(OPENSSL_VERSION_TEXT)));
+#endif
 #endif
     return devString;
 }
@@ -331,7 +361,7 @@ QVariant Theme::customMedia( CustomMediaType type )
     return re;
 }
 
-QIcon Theme::syncStateIcon( SyncResult::Status status, bool sysTray ) const
+QIcon Theme::syncStateIcon( SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible ) const
 {
     // FIXME: Mind the size!
     QString statusIcon;
@@ -363,7 +393,7 @@ QIcon Theme::syncStateIcon( SyncResult::Status status, bool sysTray ) const
         statusIcon = QLatin1String("state-error");
     }
 
-    return themeIcon( statusIcon, sysTray );
+    return themeIcon( statusIcon, sysTray, sysTrayMenuVisible );
 }
 
 QIcon Theme::folderDisabledIcon( ) const
@@ -371,9 +401,9 @@ QIcon Theme::folderDisabledIcon( ) const
     return themeIcon( QLatin1String("state-pause") );
 }
 
-QIcon Theme::folderOfflineIcon(bool systray) const
+QIcon Theme::folderOfflineIcon(bool sysTray, bool sysTrayMenuVisible ) const
 {
-    return themeIcon( QLatin1String("state-offline"), systray );
+    return themeIcon( QLatin1String("state-offline"), sysTray, sysTrayMenuVisible );
 }
 
 QColor Theme::wizardHeaderTitleColor() const
@@ -397,7 +427,7 @@ QPixmap Theme::wizardHeaderBanner() const
     if (!c.isValid())
         return QPixmap();
 
-    QPixmap pix(QSize(600, 78));
+    QPixmap pix(QSize(750, 78));
     pix.fill(wizardHeaderBackgroundColor());
     return pix;
 }
@@ -463,4 +493,6 @@ QString Theme::quotaBaseFolder() const
 {
     return QLatin1String("/");
 }
+
+
 } // end namespace client
